%% Create the environment

% Fix the random generator seed for reproducibility
rng(0)

% Create grid world
env = rlPredefinedEnv("CartPole-Discrete")

%% RL representation (neural network)

% Neural network (two inputs, the state and action, and one output)
statePath = [
    imageInputLayer([4 1 1],'Normalization','none','Name','state')
    fullyConnectedLayer(24,'Name','CriticStateFC1')
    reluLayer('Name','CriticRelu1')
    fullyConnectedLayer(24,'Name','CriticStateFC2')];

actionPath = [
    imageInputLayer([1 1 1],'Normalization','none','Name','action')
    fullyConnectedLayer(24,'Name','CriticActionFC1')];

commonPath = [
    additionLayer(2,'Name','add')
    reluLayer('Name','CriticCommonRelu')
    fullyConnectedLayer(1,'Name','output')];

dqnNetwork = layerGraph(statePath);
dqnNetwork = addLayers(dqnNetwork, actionPath);
dqnNetwork = addLayers(dqnNetwork, commonPath);    
dqnNetwork = connectLayers(dqnNetwork,'CriticStateFC2','add/in1');
dqnNetwork = connectLayers(dqnNetwork,'CriticActionFC1','add/in2');

% figure
% plot(dqnNetwork)

% representation
criticOpts = rlRepresentationOptions('LearnRate',0.01,'GradientThreshold',1); % gradient clipping
obsInfo = getObservationInfo(env);
actInfo = getActionInfo(env);
critic = rlQValueRepresentation(dqnNetwork,obsInfo,actInfo,'Observation',{'state'},'Action',{'action'},criticOpts);

%% Training options

trainOpts = rlTrainingOptions(...
    'MaxEpisodes', 1000, ...
    'MaxStepsPerEpisode', 500, ...
    'Verbose', false, ...
    'Plots','training-progress', ...
    'StopTrainingCriteria','AverageReward',...
    'StopTrainingValue',480);

% trainOpts.Plots = "none"; % turn off Episode Manager

%% Agent options
agentOpts = rlDQNAgentOptions(...
    'UseDoubleDQN',false, ...    
    'TargetUpdateMethod',"periodic", ...
    'TargetUpdateFrequency',4, ...
    'ExperienceBufferLength',100000, ...
    'DiscountFactor',0.99, ...
    'MiniBatchSize',256);

%% DQN training

dqnAagent = rlDQNAgent(critic,agentOpts);
trainingStats = train(dqnAagent,env,trainOpts);

%% Training results

plot_results({trainingStats.AverageReward}, {'dqn'}, 'dqn.png');