%% Create the environment

% Fix the random generator seed for reproducibility
rng(0)

% Create grid world
env = rlPredefinedEnv("BasicGridWorld");
% plot(env) % display environment

% Initial state
env.ResetFcn = @() 2;

%% RL representation (Q-table)

% Create the Q-table
qTable = rlTable(getObservationInfo(env),getActionInfo(env));
qRepresentation = rlQValueRepresentation(qTable,getObservationInfo(env),getActionInfo(env));

% Learning rate
qRepresentation.Options.LearnRate = 0.1; % default 0.01

%% Training options

trainOpts = rlTrainingOptions;

trainOpts.MaxStepsPerEpisode = 50;
trainOpts.MaxEpisodes = 150;
trainOpts.ScoreAveragingWindowLength = 30;
% trainOpts.StopTrainingCriteria = "AverageReward";
% trainOpts.StopTrainingValue = 11;

trainOpts.Plots = "none"; % turn off Episode Manager

%% Agent options

agentOpts = rlQAgentOptions;

agentOpts.DiscountFactor = 0.99; % default 0.99
agentOpts.EpsilonGreedyExploration.Epsilon = 0.5; % default 1.0
agentOpts.EpsilonGreedyExploration.EpsilonDecay = 0.005; % default 0.005
agentOpts.EpsilonGreedyExploration.EpsilonMin = 0.01; % default 0.01

%% Q-learning training

numRepeats = 1;
episodeRewards = zeros(trainOpts.MaxEpisodes, numRepeats);
for i = 1:numRepeats
    fprintf('Repetition %d started\n', i);
    qAgent = rlQAgent(qRepresentation,agentOpts);
    trainingStats = train(qAgent,env,trainOpts);
    episodeRewards(:,i) = trainingStats.EpisodeReward;
    % learntQTable = getLearnableParameters(getCritic(qAgent));
end
fprintf('Training ended\n');

% episodeRewards_l01_g099_e05_d0005 = episodeRewards;
% episodeRewards_l001_g099_e05_d0005 = episodeRewards;
% episodeRewards_l01_g02_e05_d0005 = episodeRewards;
% episodeRewards_l01_g099_e05_d000001 = episodeRewards;

%% Training results

% plot_results({episodeRewards_l01_g099_e05_d0005, episodeRewards_l001_g099_e05_d0005}, {'lr 0.1', 'lr 0.01'}, 'role_learning_rate.png');
% plot_results({episodeRewards_l01_g099_e05_d0005, episodeRewards_l01_g02_e05_d0005}, {'df 0.99', 'df 0.2'}, 'role_discount_factor.png');
% plot_results({episodeRewards_l01_g099_e05_d0005, episodeRewards_l01_g099_e05_d000001}, {'ed 0.005', 'ed 0.00001'}, 'role_exploration_decay.png');

%% Evaluation

% plot(env)
% env.Model.Viewer.ShowTrace = true;
% env.Model.Viewer.clearTrace;
% sim(qAgent,env)
