%% Create the environment

% Fix the random generator seed for reproducibility
rng(0)

% Create grid world
env = rlPredefinedEnv("BasicGridWorld");
% plot(env) % display environment

% Initial state
env.ResetFcn = @() 2;

%% RL representation (Q-table)

% Create the Q-table
qTable = rlTable(getObservationInfo(env),getActionInfo(env));
qRepresentation = rlQValueRepresentation(qTable,getObservationInfo(env),getActionInfo(env));

% Learning rate
qRepresentation.Options.LearnRate = 0.1; % default 0.01

%% Training options

trainOpts = rlTrainingOptions;

trainOpts.MaxStepsPerEpisode = 50;
trainOpts.MaxEpisodes = 150;
trainOpts.ScoreAveragingWindowLength = 30;
% trainOpts.StopTrainingCriteria = "AverageReward";
% trainOpts.StopTrainingValue = 11;

% trainOpts.Plots = "none"; % turn off Episode Manager

%% Agent options

agentOpts = rlSARSAAgentOptions;

agentOpts.DiscountFactor = 0.99; % default 0.99
agentOpts.EpsilonGreedyExploration.Epsilon = 0.5; % default 1.0
agentOpts.EpsilonGreedyExploration.EpsilonDecay = 0.005; % default 0.005
agentOpts.EpsilonGreedyExploration.EpsilonMin = 0.01; % default 0.01

%% SARSA training

numRepeats = 1;
episodeRewards = zeros(trainOpts.MaxEpisodes, numRepeats);
for i = 1:numRepeats
    fprintf('Repetition %d started\n', i);
    sarsaAgent = rlSARSAAgent(qRepresentation,agentOpts);
    trainingStats = train(sarsaAgent,env,trainOpts);
    episodeRewards(:,i) = trainingStats.EpisodeReward;
end
fprintf('Training ended\n');

%% Training results

% plot_results({episodeRewards}, {'sarsa'}, 'sarsa.png');